@@ -40,13 +40,14 @@
 #include "ignition/fuel_tools/ClientConfig.hh"
 #include "ignition/fuel_tools/CollectionIdentifier.hh"
 #include "ignition/fuel_tools/FuelClient.hh"
+#include "ignition/fuel_tools/Helpers.hh"
 #include "ignition/fuel_tools/JSONParser.hh"
 #include "ignition/fuel_tools/LocalCache.hh"
 #include "ignition/fuel_tools/ModelIdentifier.hh"
-#include "ignition/fuel_tools/ModelIterPrivate.hh"
+#include "ModelIterPrivate.hh"
 #include "ignition/fuel_tools/RestClient.hh"
 #include "ignition/fuel_tools/WorldIdentifier.hh"
-#include "ignition/fuel_tools/WorldIterPrivate.hh"
+#include "WorldIterPrivate.hh"
 
 using namespace ignition;
 using namespace fuel_tools;
@@ -71,7 +72,9 @@ class ignition::fuel_tools::FuelClientPrivate
     // Name
     "([^\\/]+)\\/*"
     // Version
-    "([0-9]*|tip)"};
+    "([0-9]*|tip)"
+    // Trailing /
+    "/?"};
 
   /// \brief A world URL,
   /// E.g.: https://fuel.ignitionrobotics.org/1.0/OpenRobotics/worlds/Empty/1
@@ -90,7 +93,9 @@ class ignition::fuel_tools::FuelClientPrivate
     // Name
     "([^\\/]+)\\/*"
     // Version
-    "([0-9]*|tip)"};
+    "([0-9]*|tip)"
+    // Trailing /
+    "/?"};
 
   /// \brief A model file URL,
   /// E.g.: https://server.org/1.0/owner/models/modelname/files/meshes/mesh.dae
@@ -113,7 +118,9 @@ class ignition::fuel_tools::FuelClientPrivate
     // "files"
     "files\\/+"
     // File path
-    "(.*)"};
+    "(.*)"
+    // Trailing /
+    "/?"};
 
   /// \brief A world file URL,
   /// E.g.: https://server.org/1.0/owner/worlds/worldname/files/meshes/mesh.dae
@@ -136,7 +143,9 @@ class ignition::fuel_tools::FuelClientPrivate
     // "files"
     "files\\/+"
     // File path
-    "(.*)"};
+    "(.*)"
+    // Trailing /
+    "/?"};
 
   /// \brief A collection URL,
   /// E.g.:
@@ -167,10 +176,12 @@ class ignition::fuel_tools::FuelClientPrivate
   /// \param[in] _pathToModelDir Path to the model directory.
   /// \param[in] _id Model identifier information.
   /// \param[in] _private True if this model should be private.
-  /// \param[in] _form Form to fill.
+  /// \param[in] _owner Model owner name.
+  /// \param[out] _form Form to fill.
   /// \return True if the operation completed successfully.
   public: bool FillModelForm(const std::string &_pathToModelDir,
               const ModelIdentifier &_id, bool _private,
+              const std::string &_owner,
               std::multimap<std::string, std::string> &_form);
 
   /// \brief This function requests the available licenses from the
@@ -217,23 +228,31 @@ class ignition::fuel_tools::FuelClientPrivate
 
 //////////////////////////////////////////////////
 FuelClient::FuelClient()
-  : FuelClient(ClientConfig(), Rest(), nullptr)
+  : FuelClient(ClientConfig(), Rest())
 {
 }
 
 //////////////////////////////////////////////////
-FuelClient::FuelClient(const ClientConfig &_config, const Rest &_rest,
-    LocalCache *_cache)
-  : dataPtr(new FuelClientPrivate)
+FuelClient::FuelClient(const ClientConfig &_config, const Rest &_rest)
+  : dataPtr(std::make_unique<FuelClientPrivate>())
 {
   this->dataPtr->config = _config;
   this->dataPtr->rest = _rest;
   this->dataPtr->rest.SetUserAgent(this->dataPtr->config.UserAgent());
 
-  if (nullptr == _cache)
-    this->dataPtr->cache.reset(new LocalCache(&(this->dataPtr->config)));
-  else
-    this->dataPtr->cache.reset(_cache);
+#ifndef _WIN32
+# pragma GCC diagnostic push
+# pragma GCC diagnostic ignored "-Wdeprecated-declarations"
+#else
+# pragma warning(push)
+# pragma warning(disable: 4996)
+#endif
+  this->dataPtr->cache = std::make_unique<LocalCache>(&(this->dataPtr->config));
+#ifndef _WIN32
+# pragma GCC diagnostic pop
+#else
+# pragma warning(pop)
+#endif
 
   this->dataPtr->urlModelRegex.reset(new std::regex(
     this->dataPtr->kModelUrlRegexStr));
@@ -247,6 +266,14 @@ FuelClient::FuelClient(const ClientConfig &_config, const Rest &_rest,
     this->dataPtr->kCollectionUrlRegexStr));
 }
 
+//////////////////////////////////////////////////
+FuelClient::FuelClient(const ClientConfig &_config, const Rest &_rest,
+      LocalCache *_cache) : FuelClient(_config, _rest)
+{
+  if (_cache != nullptr)
+    this->dataPtr->cache.reset(_cache);
+}
+
 //////////////////////////////////////////////////
 FuelClient::~FuelClient()
 {
@@ -320,6 +347,14 @@ ModelIter FuelClient::Models(const ServerConfig &_server) const
 Result FuelClient::WorldDetails(const WorldIdentifier &_id,
     WorldIdentifier &_world) const
 {
+
+  return this->WorldDetails(_id, _world, {});
+}
+
+//////////////////////////////////////////////////
+Result FuelClient::WorldDetails(const WorldIdentifier &_id,
+    WorldIdentifier &_world, const std::vector<std::string> &_headers) const
+{
   auto serverUrl = _id.Server().Url().Str();
 
   if (serverUrl.empty() || _id.Owner().empty() || _id.Name().empty())
@@ -332,6 +367,10 @@ Result FuelClient::WorldDetails(const WorldIdentifier &_id,
   common::URIPath path;
   path = path / _id.Owner() / "worlds" / _id.Name();
 
+  std::vector<std::string> headersIncludingServerConfig = _headers;
+  AddServerConfigParametersToHeaders(
+    _id.Server(), headersIncludingServerConfig);
+
   resp = rest.Request(HttpMethod::GET, serverUrl, version,
       path.Str(), {}, {}, "");
   if (resp.statusCode != 200)
@@ -434,13 +473,24 @@ WorldIter FuelClient::Worlds(const CollectionIdentifier &_id) const
 Result FuelClient::UploadModel(const std::string &_pathToModelDir,
     const ModelIdentifier &_id, const std::vector<std::string> &_headers,
     bool _private)
+{
+  return this->UploadModel(_pathToModelDir, _id, _headers, _private, "");
+}
+
+//////////////////////////////////////////////////
+Result FuelClient::UploadModel(const std::string &_pathToModelDir,
+    const ModelIdentifier &_id, const std::vector<std::string> &_headers,
+    bool _private, const std::string &_owner)
 {
   ignition::fuel_tools::Rest rest;
   RestResponse resp;
 
   std::multimap<std::string, std::string> form;
-  if (!this->dataPtr->FillModelForm(_pathToModelDir, _id, _private, form))
+  if (!this->dataPtr->FillModelForm(_pathToModelDir, _id, _private, _owner,
+        form))
+  {
     return Result(ResultType::UPLOAD_ERROR);
+  }
 
   std::vector<std::string> headersIncludingServerConfig = _headers;
   AddServerConfigParametersToHeaders(
@@ -452,10 +502,26 @@ Result FuelClient::UploadModel(const std::string &_pathToModelDir,
 
   if (resp.statusCode != 200)
   {
+    std::string categories;
+    if (form.find("categories") != form.end())
+    {
+      categories = form.find("categories")->second;
+    }
+
     ignerr << "Failed to upload model." << std::endl
            << "  Server: " << _id.Server().Url().Str() << std::endl
+           << "  Server API Version: " <<  _id.Server().Version() << std::endl
            << "  Route: /models\n"
-           << "  REST response code: " << resp.statusCode << std::endl;
+           << "  Categories: " << categories << std::endl
+           << "  REST response code: " << resp.statusCode
+           << std::endl << std::endl
+           << "Suggestions" << std::endl
+           << "  1. Is the Server URL correct? Try entering it on a browser.\n"
+           << "  2. Do the categories exist? If you are using the Fuel server,"
+           << "     then you can get the complete list at"
+           << "     https://fuel.ignitionrobotics.org/1.0/categories.\n"
+           << "  3. If the owner is specified, make sure you have correct\n"
+           << "     permissions." << std::endl;
     return Result(ResultType::FETCH_ERROR);
   }
 
@@ -680,6 +746,13 @@ Result FuelClient::DownloadModel(const ModelIdentifier &_id,
 
 //////////////////////////////////////////////////
 Result FuelClient::DownloadWorld(WorldIdentifier &_id)
+{
+  return this->DownloadWorld(_id, {});
+}
+
+//////////////////////////////////////////////////
+Result FuelClient::DownloadWorld(WorldIdentifier &_id,
+    const std::vector<std::string> &_headers)
 {
   // Server config
   if (!_id.Server().Url().Valid() || _id.Server().Version().empty())
@@ -696,7 +769,7 @@ Result FuelClient::DownloadWorld(WorldIdentifier &_id)
 
   ignmsg << "Downloading world [" << _id.UniqueName() << "]" << std::endl;
 
-  std::vector<std::string> headersIncludingServerConfig;
+  std::vector<std::string> headersIncludingServerConfig = _headers;
   AddServerConfigParametersToHeaders(
     _id.Server(), headersIncludingServerConfig);
 
@@ -762,8 +835,6 @@ bool FuelClient::ParseModelUrl(const common::URI &_modelUrl,
   std::string modelName;
   std::string modelVersion;
 
-  std::regex_match(urlStr, match, *this->dataPtr->urlModelRegex);
-
   if (std::regex_match(urlStr, match, *this->dataPtr->urlModelRegex) &&
       match.size() >= 5u)
   {
@@ -830,8 +901,6 @@ bool FuelClient::ParseWorldUrl(const common::URI &_worldUrl,
   std::string worldName;
   std::string worldVersion;
 
-  std::regex_match(urlStr, match, *this->dataPtr->urlWorldRegex);
-
   if (std::regex_match(urlStr, match, *this->dataPtr->urlWorldRegex) &&
       match.size() >= 5u)
   {
@@ -1305,7 +1374,7 @@ Result FuelClient::PatchModel(
 
   if (!_pathToModelDir.empty() &&
       !this->dataPtr->FillModelForm(_pathToModelDir, _model,
-        _model.Private(), form))
+        _model.Private(), _model.Owner(), form))
   {
     return Result(ResultType::UPLOAD_ERROR);
   }
@@ -1351,7 +1420,7 @@ void FuelClient::PopulateLicenses(const ServerConfig &_server)
 
 //////////////////////////////////////////////////
 bool FuelClientPrivate::FillModelForm(const std::string &_pathToModelDir,
-    const ModelIdentifier &_id, bool _private,
+    const ModelIdentifier &_id, bool _private, const std::string &_owner,
     std::multimap<std::string, std::string> &_form)
 {
   if (!common::exists(_pathToModelDir))
@@ -1408,6 +1477,12 @@ bool FuelClientPrivate::FillModelForm(const std::string &_pathToModelDir,
     {"private", _private ? "1" : "0"},
   };
 
+  // Add owner if specified.
+  if (!_owner.empty())
+  {
+    _form.emplace("owner", _owner);
+  }
+
   // \todo(nkoenig) The ign-fuelserver expects an integer number for the
   // license information. The fuelserver should be modified to accept
   // a string. Otherwise, we have to bake into each client a mapping of
@@ -1470,7 +1545,6 @@ bool FuelClientPrivate::FillModelForm(const std::string &_pathToModelDir,
     _form.emplace("tags", tags);
 
   // Add categories
-  /* \todo: Uncomment in version 5.0
   std::string categories;
   if (meta.has_categories())
   {
@@ -1489,7 +1563,6 @@ bool FuelClientPrivate::FillModelForm(const std::string &_pathToModelDir,
   }
   if (!categories.empty())
     _form.emplace("categories", categories);
-  */
 
   // Add annotations as metadata.
   for (const auto &annotation : meta.annotations())
@@ -1529,3 +1602,85 @@ void FuelClientPrivate::PopulateLicenses(const ServerConfig &_server)
     ignerr << "Failed to parse license information[" << resp.data << "]\n";
   }
 }
+
+//////////////////////////////////////////////////
+bool FuelClient::UpdateModels(const std::vector<std::string> &_headers)
+{
+  // Get a list of the most recent model versions in the cache.
+  std::map<std::string, ignition::fuel_tools::ModelIdentifier> toProcess;
+  for (ModelIter iter = this->dataPtr->cache->AllModels(); iter; ++iter)
+  {
+    auto processIter = toProcess.find(iter->Identification().UniqueName());
+    if (processIter == toProcess.end() ||
+        processIter->second.Version() < iter->Identification().Version())
+    {
+      toProcess[iter->Identification().UniqueName()] = iter->Identification();
+    }
+  }
+
+  // Attempt to update each model.
+  for (const auto id : toProcess)
+  {
+    ignition::fuel_tools::ModelIdentifier cloudId;
+
+    if (!this->ModelDetails(id.second, cloudId, _headers))
+    {
+      ignerr << "Failed to fetch model details for model["
+        << id.second.Owner()  << "/" << id.second.Name() << "]\n";
+    }
+    else if (id.second.Version() < cloudId.Version())
+    {
+      ignmsg << "Updating model " << id.second.Owner() << "/"
+        << id.second.Name() << " up to version "
+        << cloudId.Version() << std::endl;
+      this->DownloadModel(cloudId, _headers);
+    }
+    else
+    {
+      ignmsg << "Model " << id.second.Owner() << "/"
+        << id.second.Name() << " is up to date." << std::endl;
+    }
+  }
+  return true;
+}
+
+//////////////////////////////////////////////////
+bool FuelClient::UpdateWorlds(const std::vector<std::string> &_headers)
+{
+  // Get a list of the most recent world versions in the cache.
+  std::map<std::string, ignition::fuel_tools::WorldIdentifier> toProcess;
+  for (WorldIter iter = this->dataPtr->cache->AllWorlds(); iter; ++iter)
+  {
+    auto processIter = toProcess.find(iter->UniqueName());
+    if (processIter == toProcess.end() ||
+        processIter->second.Version() < iter->Version())
+    {
+      toProcess[iter->UniqueName()] = *iter;
+    }
+  }
+
+  // Attempt to update each world.
+  for (const auto id : toProcess)
+  {
+    ignition::fuel_tools::WorldIdentifier cloudId;
+
+    if (!this->WorldDetails(id.second, cloudId, _headers))
+    {
+      ignerr << "Failed to fetch world details for world["
+        << id.second.Owner() << "/" << id.second.Name() << "]\n";
+    }
+    else if (id.second.Version() < cloudId.Version())
+    {
+      ignmsg << "Updating world " << id.second.Owner() << "/"
+        << id.second.Name() << " up to version "
+        << cloudId.Version() << std::endl;
+      this->DownloadWorld(cloudId, _headers);
+    }
+    else
+    {
+      ignmsg << "World " << id.second.Owner() << "/"
+        << id.second.Name() << " is up to date." << std::endl;
+    }
+  }
+  return true;
+}
