@@ -35,11 +35,13 @@
 #include <ignition/math/SemanticVersion.hh>
 
 #include "ignition/fuel_tools/ClientConfig.hh"
+#include "ignition/fuel_tools/Helpers.hh"
 #include "ignition/fuel_tools/LocalCache.hh"
-#include "ignition/fuel_tools/ModelIterPrivate.hh"
-#include "ignition/fuel_tools/ModelPrivate.hh"
 #include "ignition/fuel_tools/Zip.hh"
-#include "ignition/fuel_tools/WorldIterPrivate.hh"
+
+#include "ModelPrivate.hh"
+#include "ModelIterPrivate.hh"
+#include "WorldIterPrivate.hh"
 
 using namespace ignition;
 using namespace fuel_tools;
@@ -231,7 +233,8 @@ ModelIter LocalCache::AllModels()
     for (auto &server : this->dataPtr->config->Servers())
     {
       std::string path = common::joinPaths(
-          this->dataPtr->config->CacheLocation(), server.Url().Path().Str());
+          this->dataPtr->config->CacheLocation(), uriToPath(server.Url()));
+
       auto srvModels = this->dataPtr->ModelsInServer(path);
       for (auto &mod : srvModels)
       {
@@ -254,7 +257,7 @@ WorldIter LocalCache::AllWorlds() const
     for (auto &server : this->dataPtr->config->Servers())
     {
       std::string path = common::joinPaths(
-          this->dataPtr->config->CacheLocation(), server.Url().Path().Str());
+          this->dataPtr->config->CacheLocation(), uriToPath(server.Url()));
 
       // Make sure the server info is correct
       auto srvWorlds = this->dataPtr->WorldsInServer(path);
@@ -386,7 +389,7 @@ bool LocalCache::SaveModel(
   std::string cacheLocation = this->dataPtr->config->CacheLocation();
 
   std::string modelRootDir = common::joinPaths(cacheLocation,
-      _id.Server().Url().Path().Str(), _id.Owner(), "models", _id.Name());
+      uriToPath(_id.Server().Url()), _id.Owner(), "models", _id.Name());
   std::string modelVersionedDir =
     common::joinPaths(modelRootDir, _id.VersionStr());
 
@@ -673,6 +676,10 @@ void LocalCachePrivate::FixPathsInMaterialElement(
             workflowElem->FirstChildElement("emissive_map");
         if (emissiveElem)
           this->FixPathsInUri(emissiveElem, _id);
+        tinyxml2::XMLElement *lightElem =
+            workflowElem->FirstChildElement("light_map");
+        if (lightElem)
+          this->FixPathsInUri(lightElem, _id);
         // metal workflow specific elements
         if (workflow == "metal")
         {
